<?xml version="1.0" encoding="UTF-8"?>

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="text"/>

<xsl:include href="util.xslt"/>

<xsl:template name="build-varname">
	<xsl:param name="form-name"/>
	<xsl:param name="input-name"/>
	<xsl:param name="input-value"/>
	<xsl:value-of select="$form-name"/>
	<xsl:text>_</xsl:text>
	<xsl:value-of select="$input-name"/>
	<xsl:if test="string-length($input-value) > 0">
		<xsl:text>_</xsl:text>
		<xsl:value-of select="$input-value"/>
	</xsl:if>
</xsl:template>

<xsl:template name="_build-onclick">
	<xsl:param name="list"/>
	<xsl:param name="boolean"/>
	<xsl:choose>
		<xsl:when test="contains($list, ',')">
			<xsl:value-of select="substring-before($list, ',')"/>
			<xsl:text>.disabled = </xsl:text>
			<xsl:value-of select="$boolean"/>
			<xsl:text>;</xsl:text>
			<xsl:call-template name="_build-onclick">
				<xsl:with-param name="list" select="substring-after($list, ',')"/>
				<xsl:with-param name="boolean" select="$boolean"/>
			</xsl:call-template>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="$list"/>
			<xsl:text>.disabled = </xsl:text>
			<xsl:value-of select="$boolean"/>
			<xsl:text>;</xsl:text>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="build-onclick">

	<xsl:param name="enables"/>
	<xsl:param name="disables"/>
	<xsl:param name="onclick"/>

	<xsl:text>\" onclick=\"if (document.</xsl:text>
	<xsl:value-of select="../@name"/>
	<xsl:text>.</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>[</xsl:text>
	<xsl:value-of select="position() - 1"/>
	<xsl:text>].checked == 1) { </xsl:text>
	<xsl:if test="string-length($enables) > 0">
		<xsl:call-template name="_build-onclick">
			<xsl:with-param name="list" select="$enables"/>
			<xsl:with-param name="boolean" select="'false'"/>
		</xsl:call-template>
	</xsl:if>
	<xsl:if test="string-length($disables) > 0">
		<xsl:call-template name="_build-onclick">
			<xsl:with-param name="list" select="$disables"/>
			<xsl:with-param name="boolean" select="'true'"/>
		</xsl:call-template>
	</xsl:if>
	<xsl:text>} else {</xsl:text>
	<xsl:text>}</xsl:text>
	<xsl:if test="string-length($onclick) > 0">
		<xsl:value-of select="$onclick"/>
		<xsl:text>();</xsl:text>
	</xsl:if>
</xsl:template>

<xsl:template match="webapp">
	<xsl:text>/* WARNING: This C source code was generated from XML: DO NOT EDIT BY HAND! */</xsl:text>
	<xsl:value-of select="$cr"/>
	<xsl:value-of select="$cr"/>
	<xsl:text>
#include &lt;stdio.h&gt;
#include &lt;stdio.h&gt;
#include &lt;string.h&gt;
#include &lt;stdlib.h&gt;
#include &lt;unistd.h&gt;
#include &lt;sys/types.h&gt;
#include &lt;sys/stat.h&gt;
#include &lt;cgi-util.h&gt;
</xsl:text>
	<xsl:apply-templates select="(fieldset|form)" mode="pound-statements"/>
	<xsl:value-of select="$cr"/>
	<xsl:text>
void print_form (char *sidebar)
{
	struct stat buf;</xsl:text>
	<xsl:value-of select="$cr"/>
	<xsl:apply-templates select="(fieldset/form/input|form/input|fieldset/form/select|form/select)" mode="declare-vars"/>
	<xsl:text>printf("</xsl:text>
	<xsl:apply-templates select="top"/>
	<xsl:text>", sidebar);
</xsl:text>
	<xsl:apply-templates select="(fieldset|form)" mode="print-form"/>
	<xsl:text>printf("</xsl:text>
	<xsl:apply-templates select="bottom"/>
	<xsl:text>");
}

void save_data(void)
{
	FILE *conffile;
	struct stat buf;

	</xsl:text>
	<xsl:apply-templates select="(fieldset/form/input|form/input|fieldset/form/select|form/select)" mode="declare-vars"/>
	<xsl:apply-templates select="(fieldset|form)" mode="save-input"/>
	<xsl:text>
}

char *get_service_name(void)
{
        return g_strdup("</xsl:text><xsl:value-of select="@name"/><xsl:text>foo");
}</xsl:text>
</xsl:template>

<xsl:template match="fieldset" mode="pound-statements">
	<xsl:apply-templates select="form" mode="pound-statements"/>
</xsl:template>

<xsl:template match="fieldset" mode="print-form">
	<xsl:text>printf("&lt;fieldset&gt;");</xsl:text>
	<xsl:text>printf("&lt;legend&gt;");</xsl:text>
	<xsl:text>printf("</xsl:text>
	<xsl:value-of select="legend"/>
	<xsl:text>");</xsl:text>
	<xsl:text>printf("&lt;/legend&gt;");</xsl:text>
	<xsl:apply-templates select="form" mode="print-form"/>
	<xsl:text>printf("&lt;/fieldset&gt;");</xsl:text>
</xsl:template>

<xsl:template match="form" mode="pound-statements">
	<xsl:text>#define CONFFILE_</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text> "/tmp/</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>"
</xsl:text>
</xsl:template>

<xsl:template match="form" mode="print-form">
	<xsl:text>
	printf("&lt;p&gt;&lt;form name=\"</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>\" action=\"/cgi-bin/appliance-config-</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>.cgi\" method=\"post\"&gt;");
	if (stat (CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, &amp;buf) == -1) {</xsl:text>
		<xsl:apply-templates select="*" mode="output-no-stored-settings"/>
	<xsl:text>
	} else { /* FIXME: need conf parser lib. Unfortunately, glib requires [GROUPS] */
		FILE *conffile;

		char buf[BUFSIZ];
		if ((conffile = fopen (CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, "r")) == NULL) {
			cgi_error ("Error opening " CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>);
		}

		while (fgets (buf, sizeof (buf), conffile) != NULL) {
			buf[strlen (buf) - 1] = 0x00; /* get rid of '\n' */

	</xsl:text>
	<xsl:apply-templates select="*" mode="read-stored-settings"/>
	<xsl:text>
		}
	</xsl:text>
	<xsl:apply-templates select="*" mode="output-stored-settings"/>
	<xsl:text>
		if (fclose (conffile) == EOF) {
			cgi_error ("Error closing " CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>);
		}
	}
	</xsl:text>
	<xsl:apply-templates select="submit"/>
	<xsl:text>
	printf("&lt;/form&gt;&lt;p&gt;\n");
	</xsl:text>
</xsl:template>

<xsl:template match="fieldset|form" mode="save-input">
	<xsl:text>
	if (stat (CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, &amp;buf) == 0) {
		/* Deleted by appliance-configd after running all scripts */
		if (rename (CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text> ".old") == -1) {
			cgi_error ("Error renaming old configuration");
		}
	}
	if ((conffile = fopen (CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>, "w")) == NULL) {
		cgi_error ("Error opening " CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>);
	}
	fprintf (conffile, "# Created by appliance-config, do not edit by hand\n");
	fprintf (conffile, "\n");
	</xsl:text>
	<xsl:apply-templates select="(form/input|form/select|input|select)" mode="save-input"/>
	<xsl:text>
	if (fclose (conffile) == EOF) {
		cgi_error ("Error closing " CONFFILE_</xsl:text><xsl:value-of select="@name"/><xsl:text>);
	}
	</xsl:text>
</xsl:template>

<xsl:template match="input|select" mode="declare-vars">
	<xsl:text>	char *</xsl:text>
	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>
	<xsl:text> = NULL;</xsl:text>
</xsl:template>

<xsl:template match="input|select" mode="save-input">
	<xsl:text>
	if ((</xsl:text>
	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>
	<xsl:text> = cgi_getentrystr ("</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>")) == NULL || *</xsl:text>
	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>
	<xsl:text> == 0x00) {
		</xsl:text>
	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>
	<xsl:text> = "off";
	}
	fprintf (conffile, "</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>=%s\n", </xsl:text>
	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>
	<xsl:text>);
	</xsl:text>
</xsl:template>

<xsl:template match="input" mode="output-no-stored-settings">
	<xsl:text>
		printf("</xsl:text>
	<xsl:value-of select="@label"/>
	<xsl:text>&lt;input name=\"</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>\" type=\"</xsl:text>
	<xsl:value-of select="@type"/>
	<xsl:if test="@enables or @disables or @onclick">
		<xsl:call-template name="build-onclick">
                        <xsl:with-param name="enables" select="@enables"/>
                        <xsl:with-param name="disables" select="@disables"/>
                        <xsl:with-param name="onclick" select="@onclick"/>
                </xsl:call-template>
	</xsl:if>
	<xsl:text>\"/&gt;");</xsl:text>
</xsl:template>

<xsl:template match="input" mode="read-stored-settings">
	<xsl:text>
			if (g_ascii_strncasecmp (buf, "</xsl:text>

	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>
	<xsl:text>=", strlen ("</xsl:text>

	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>

	<xsl:text>=")) == 0) {</xsl:text>
	<xsl:value-of select="$cr"/>
	<xsl:value-of select="$tab"/>
	<xsl:value-of select="$tab"/>
	<xsl:value-of select="$tab"/>
	<xsl:value-of select="$tab"/>

	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>

	<xsl:text> = g_strdup (buf + strlen ("</xsl:text>

	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>

	<xsl:text>="));
			}</xsl:text>
</xsl:template>

<xsl:template match="input" mode="output-stored-settings">
	<xsl:text>
			if (g_ascii_strcasecmp ("on", </xsl:text>

	<xsl:call-template name="build-varname">
		<xsl:with-param name="form-name" select="../@name"/>
		<xsl:with-param name="input-name" select="@name"/>
		<xsl:with-param name="input-value" select="@value"/>
	</xsl:call-template>

	<xsl:text>) == 0) {
				printf("</xsl:text>
		<xsl:value-of select="@label"/>
		<xsl:text>&lt;input name=\"</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>\" type=\"</xsl:text>
		<xsl:value-of select="@type"/>
		<xsl:text>\" checked=\"checked\" /&gt;");
			} else {
				printf("&lt;input name=\"</xsl:text>
		<xsl:value-of select="@name"/>
		<xsl:text>\" type=\"</xsl:text>
		<xsl:value-of select="@type"/>
		<xsl:text>\"/&gt;");
			}</xsl:text>
</xsl:template>

<xsl:template match="select" mode="output-no-stored-settings">
	<xsl:text>
		printf("</xsl:text>
	<xsl:value-of select="@label"/>
	<xsl:text>&lt;select name=\"</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>\"&gt;");</xsl:text>
	<xsl:apply-templates select="*" mode="output-no-stored-settings"/>
	<xsl:text>
		printf("&lt;/select&gt;");
	</xsl:text>
</xsl:template>

<xsl:template match="select" mode="output-stored-settings">
	<xsl:text>
		printf("</xsl:text>
	<xsl:value-of select="@label"/>
	<xsl:text>&lt;select name=\"</xsl:text>
	<xsl:value-of select="@name"/>
	<xsl:text>\"&gt;");</xsl:text>
	<xsl:apply-templates select="*" mode="output-stored-settings"/>
	<xsl:text>
		printf("&lt;/select&gt;");
	</xsl:text>
</xsl:template>

<xsl:template match="select" mode="read-stored-settings">
</xsl:template>

<xsl:template match="option" mode="output-no-stored-settings">
	<xsl:text>
		printf("&lt;option value=\"</xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text>\"&gt;</xsl:text>
	<xsl:value-of select="."/>
	<xsl:text>&lt;/option&gt;");</xsl:text>
</xsl:template>

<xsl:template match="option" mode="output-stored-settings">
	<xsl:text>
		printf("&lt;option value=\"</xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text>\"&gt;</xsl:text>
	<xsl:value-of select="."/>
	<xsl:text>&lt;/option&gt;");</xsl:text>
</xsl:template>

<xsl:template match="call-print-options-fn" mode="output-no-stored-settings">
	<xsl:value-of select="."/>
	<xsl:text>("document.</xsl:text>
	<xsl:value-of select="../../@name"/>
	<xsl:text>.</xsl:text>
	<xsl:value-of select="../@name"/>
	<xsl:text>");</xsl:text>
</xsl:template>

<xsl:template match="call-print-options-fn" mode="output-stored-settings">
	<xsl:value-of select="."/>
	<xsl:text>("document.</xsl:text>
	<xsl:value-of select="../../@name"/>
	<xsl:text>.</xsl:text>
	<xsl:value-of select="../@name"/>
	<xsl:text>");</xsl:text>
</xsl:template>

<xsl:template match="br" mode="output-no-stored-settings">
	<xsl:text>printf("&lt;br/&gt;");</xsl:text>
</xsl:template>

<xsl:template match="br" mode="output-stored-settings">
	<xsl:text>printf("&lt;br/&gt;");</xsl:text>
</xsl:template>

<xsl:template match="submit">
	<xsl:text>
	printf("&lt;p&gt;&lt;input name=\"submit\" type=\"submit\" value=\"</xsl:text>
	<xsl:value-of select="@value"/>
	<xsl:text>\"/&gt;&lt;p&gt;");</xsl:text>
	<xsl:value-of select="$cr"/>
</xsl:template>

</xsl:stylesheet>
